
#include "client.hpp"

#include <iostream>

#include <boost/thread.hpp>

#include <sys/socket.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>

#include "tcpinfo.hpp"

namespace {

void send_loop(const std::string& address, unsigned short port, size_t size, int num_packets, const DisplayTcpInfo& info);

}

void start_client(const std::string& address, unsigned short port, int num_sockets, int num_packets, size_t size)
{
    DisplayTcpInfo info;
    boost::thread_group send_threads;

    for (int i=0; i<num_sockets; ++i)
    {
        send_threads.add_thread(new boost::thread(send_loop, boost::cref(address), port, size, num_packets, boost::cref(info)));
    }

    send_threads.join_all();
}

namespace {

void send_loop(const std::string& address, unsigned short port, size_t size, int num_packets, const DisplayTcpInfo& info)
{
        std::string pkt('a', size);
        const char* ptr = pkt.c_str();
        size_t tot_len = 0;


    while (true)
    {
        int sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

        if (sock == -1)
        {
            std::cerr << "Error opening socket: " << std::strerror(errno) << std::endl;

            break;
        }

        sockaddr_in addr = sockaddr_in();
        addr.sin_family = AF_INET;
        addr.sin_addr.s_addr = inet_addr(address.c_str());
        addr.sin_port = htons(port);

        if (connect(sock, reinterpret_cast<sockaddr*>(&addr), sizeof(addr)) != 0)
        {
            std::cerr << "Error connecting to server: " << std::strerror(errno) << std::endl;
            close(sock);

            return;
        }

//        std::string pkt('a', size);
//        const char* ptr = pkt.c_str();
//        size_t tot_len = 0;

        for (int i=0; i<num_packets; ++i)
        {
            size_t len = 0;

            while (true)
            {
                len += send(sock, ptr + len, size - len, 0);
                tot_len += len;

        //        info.display(sock, tot_len);

                if (len == size)
                {
                    break;
                }
            }
        }

        close(sock);
	//boost::this_thread::sleep(boost::posix_time::milliseconds(1000));
    }
}

} // Anonymous namespace
